@@ -13,6 +13,15 @@ class Timeframe(enum.Enum):
     MINUTE = '1m'
 
 
+FREQUENCY = {
+    Timeframe.YEAR: 'y',
+    Timeframe.MONTH: 'm',
+    Timeframe.DAY: 'D',
+    Timeframe.HOUR: 'h',
+    Timeframe.MINUTE: '1min'
+}
+
+
 class PlotType(enum.Enum):
     CANDLE_PLOTS = 'candle_plots'
     SUB_PLOTS = 'sub_plots'
