@@ -3,8 +3,8 @@ import asyncio
 import numpy as np
 import pandas as pd
 from time import sleep
-from typing import List
-from backtest_view.constants import PlotType, Timeframe
+from typing import List, Optional
+from backtest_view.constants import PlotType, Timeframe, FREQUENCY
 from lightweight_charts import Chart, JupyterChart
 from backtest_view.utilities import within_timestamp, candles_to_seconds
 from backtest_view.data_source.abstract import AbstractDataSource
@@ -18,10 +18,13 @@ class Plot:
             self,
             data_sources: List[AbstractDataSource],
             timeframe: Timeframe,
-            replay: bool = True,
+            replay: bool = False,
             show_subplots: bool = True,
             candle_plot_scale: int = 3,
-            redraw_interval: float = 0.01
+            redraw_interval: float = 0.01,
+            width: int = 800,
+            height: int = 350,
+            isolate_pair: Optional[str] = None
     ):
         self._data_sources = data_sources
         self._timeframe = timeframe
@@ -36,14 +39,30 @@ class Plot:
         self._sub_plot_height = None
         self._candle_plot_height = None
         self._current_trade_index = {}
+        self._notebook_chart_kwargs = {
+            'width': width,
+            'height': height
+        }
+        self._isolate_pair = isolate_pair
 
         # event variables
         self._paused = False
         self._current_time_index = 0
 
+        # validate
+        self._validate()
+
+        # initialize data
         self._create_main_chart()
         self._set_dataframes()
 
+    def _validate(self):
+        if self.in_notebook and self._replay:
+            raise ValueError(
+                f'The `replay` flag is {self._replay}, but this is in a notebook environment. '
+                f'The replay feature is not available in a notebook.'
+                )
+
     def _create_main_chart(self):
         num_candle_plots = 0
         num_sub_plots = 0
@@ -59,22 +78,42 @@ class Plot:
             self._sub_plot_height = 0
             self._candle_plot_height = 1 / num_candle_plots
 
-        # switch chart class if this is run in a notebook
+        # switch chart class and kwargs if this is run in a notebook
         chart_class = Chart
+        kwargs = {
+            'inner_width': 1, 
+            'inner_height': self._candle_plot_height
+            }
+
+        # set height to 1 if isolating a pair
+        if self._isolate_pair:
+            kwargs['inner_height'] = 1
+
         if self.in_notebook:
             chart_class = JupyterChart
+            kwargs.update(self._notebook_chart_kwargs)
 
-        self._chart = chart_class(inner_width=1, inner_height=self._candle_plot_height)
+        self._chart = chart_class(**kwargs)
+
+    def _get_timerange(self, dataframe):
+        if not self.first_trade_open and not self.last_trade_close:
+            return dataframe
+
+        return dataframe[(
+                    dataframe['date'] > self.first_trade_open - self.startup_time_delta) & (
+                    dataframe['date'] < self.last_trade_close)]
 
     def _set_dataframes(self):
         for data_source in self._data_sources:
             self._dataframes[data_source.name] = self._dataframes.get(data_source.name, {})
 
             for pair, dataframe in data_source.dataframes.items():
+                # dont set data if isolating a pair
+                if self._isolate_pair and pair != self._isolate_pair:
+                    continue
+
                 # get the time range to show based on the trades
-                time_range = dataframe[(
-                    dataframe['date'] > self.first_trade_open - self.startup_time_delta) & (
-                    dataframe['date'] < self.last_trade_close)]
+                time_range = self._get_timerange(dataframe)
 
                 # if replay is off then initialize the chart with all candles
                 if not self._replay:
@@ -90,15 +129,16 @@ class Plot:
                 }
 
     def _get_timestamps(self) -> pd.DatetimeIndex:
+        # provide an offset of 2 so that the first index on the chart is populated
+        seconds = candles_to_seconds(
+            count=2,
+            timeframe=self._timeframe.value
+        )
+
         return pd.period_range(
-            start=self.first_trade_open - self.startup_time_delta + pd.Timedelta(
-                seconds=candles_to_seconds(
-                    count=2,
-                    timeframe=self._timeframe.value
-                )
-            ),
+            start=self.first_trade_open - self.startup_time_delta + pd.Timedelta(seconds=seconds),
             end=self.last_trade_close,
-            freq=self._timeframe.value[-1]
+            freq=FREQUENCY[self._timeframe]
         ).to_timestamp().tz_localize('utc')
 
     @property
@@ -127,6 +167,10 @@ class Plot:
         for data_source in self._data_sources:
             if data_source.trades:
                 for pair, trades in data_source.trades.items():
+                    # dont set data if isolating a pair
+                    if self._isolate_pair and pair != self._isolate_pair:
+                        continue
+
                     open_date = pd.Timestamp(trades[0]['open_date'])
                     if not first_trade or open_date < first_trade:
                         first_trade = open_date
@@ -138,6 +182,10 @@ class Plot:
         for data_source in self._data_sources:
             if data_source.trades:
                 for pair, trades in data_source.trades.items():
+                    # dont set data if isolating a pair
+                    if self._isolate_pair and pair != self._isolate_pair:
+                        continue
+
                     close_date = pd.Timestamp(trades[-1]['close_date'])
                     if not last_trade or close_date > last_trade:
                         last_trade = close_date
@@ -148,9 +196,13 @@ class Plot:
             series: pd.Series,
             name: str,
     ) -> pd.Series:
+        value = series[name]
+        # set null values to 0
+        if pd.isnull(value):
+            value = 0
         return pd.Series({
             'time': series['date'],
-            name: series[name]
+            name: value
         })
 
     @staticmethod
@@ -279,6 +331,12 @@ class Plot:
                 # candle plots must specify a pair
                 pair = data.get('pair', default_pair)
 
+                # dont set data if isolating a pair
+                if self._isolate_pair and pair != self._isolate_pair:
+                    continue
+                elif self._isolate_pair:
+                    index = 0
+
                 dataframe = self._dataframes[data_source.name][pair]['initial_dataframe']
                 self._references[data_source.name][pair] = self._references[data_source.name].get(pair, {})
 
@@ -322,16 +380,26 @@ class Plot:
 
         for index, candle_plot in enumerate(data_source.replay_config[plot_type]):
             pair = candle_plot['pair']
+
+            # dont set data if isolating a pair
+            if self._isolate_pair and pair != self._isolate_pair:
+                continue
+            elif self._isolate_pair:
+                index = 0
+
             trades = data_source.trades.get(pair)
             if trades:
                 trade_index = self._current_trade_index.get(pair, 0)
                 if trade_index < len(data_source.trades[pair]):
                     trade = data_source.trades[pair][trade_index]
                     trade_pair = trade['pair']
+                    is_futures_pair = len(trade['pair'].split(':')) == 2
                     trade_start_date = pd.Timestamp(trade['open_date'])
                     trade_end_date = pd.Timestamp(trade['close_date'])
                     trade_profit = trade['profit_abs']
                     trade_exit_reason = trade['exit_reason']
+                    side = 'short' if trade['is_short'] else 'long'
+                    leverage = trade['leverage']
                     trade_amount = trade['amount']
                     trade_open_rate = trade['open_rate']
                     trade_close_rate = trade['close_rate']
@@ -352,7 +420,7 @@ class Plot:
                     if within_timestamp(timestamp, trade_start_date, timeframe=self._timeframe):
                         chart.marker(
                             time=timestamp,
-                            text=f'Entered ${trade_amount * trade_open_rate} {trade_pair}',
+                            text=f'Entered {leverage}x {side} ${(trade_amount * trade_open_rate)/leverage}',
                             position='below',
                             shape='arrow_up',
                             color='#18FFFF'
@@ -362,7 +430,8 @@ class Plot:
                         prefix = '$' if trade_profit > 0 else '-$'
                         chart.marker(
                             time=timestamp,
-                            text=f'Exited ${trade_amount * trade_close_rate} {trade_pair} Profit: {prefix}{abs(trade_profit)}',
+                            text=f'Exited {leverage}x {side} ${(trade_amount * trade_close_rate)/leverage} '
+                                 f'Profit: {prefix}{abs(trade_profit)}',
                             position='above',
                             shape='arrow_down',
                             color=color
@@ -377,6 +446,10 @@ class Plot:
 
         # set trades from all data sources
         for data_source in self._data_sources:
+            # dont set data if isolating a pair
+            if self._isolate_pair and data_source.pair != self._isolate_pair:
+                continue
+
             dataframe = self._dataframes[data_source.name][data_source.pair]['initial_dataframe']
             for _, series in dataframe.iterrows():
                 self.set_trade(data_source, series['date'])
@@ -389,36 +462,51 @@ class Plot:
 
             # candle plots must specify a pair
             pair = data.get('pair', default_pair)
+
+            # dont set data if isolating a pair
+            if self._isolate_pair and pair != self._isolate_pair:
+                continue
+            elif self._isolate_pair:
+                index = 0
+
             dataframe = self._dataframes[data_source.name][pair]['replay_dataframe']
-            series = dataframe.loc[dataframe['date'] == timestamp].iloc[-1]
-            chart = self._references[data_source.name][pair][plot_type.value][index]['chart']
+            row = dataframe.loc[dataframe['date'] == timestamp]
+            if not row.empty:
+                series = row.iloc[-1]
 
-            # update candles if this is a candle plot
-            if plot_type == PlotType.CANDLE_PLOTS:
-                candle = self.candle_update(series)
-                chart.update(candle)
+                chart = self._references[data_source.name][pair][plot_type.value][index]['chart']
 
-            # update lines
-            for line_index, line_name in enumerate(data.get('lines', {}).keys()):
-                point = self.line_update(
-                    series,
-                    name=line_name
-                )
-                self._references[data_source.name][pair][plot_type.value][index]['lines'][line_index].update(point)
+                # update candles if this is a candle plot
+                if plot_type == PlotType.CANDLE_PLOTS:
+                    candle = self.candle_update(series)
+                    chart.update(candle)
 
-            # update markers
-            for marker_name, marker_data in data.get('markers', {}).items():
-                value = series[marker_name]
-                display_settings = marker_data.get(value)
-
-                if display_settings:
-                    chart.marker(
-                        time=timestamp,
-                        text=display_settings.get('text'),
-                        position=display_settings.get('position'),
-                        shape=display_settings.get('shape'),
-                        color=display_settings.get('color')
+                # update lines
+                for line_index, line_name in enumerate(data.get('lines', {}).keys()):
+                    point = self.line_update(
+                        series,
+                        name=line_name
                     )
+                    self._references[data_source.name][pair][plot_type.value][index]['lines'][line_index].update(point)
+
+                # update markers
+                for marker_name, marker_data in data.get('markers', {}).items():
+                    value = series[marker_name]
+                    display_settings = marker_data.get(value)
+
+                    if display_settings:
+                        chart.marker(
+                            time=timestamp,
+                            text=display_settings.get('text'),
+                            position=display_settings.get('position'),
+                            shape=display_settings.get('shape'),
+                            color=display_settings.get('color')
+                        )
+            else:
+                raise RuntimeError(
+                    f'Missing data for symbol {pair} on date {timestamp}. '
+                    f'Please make sure this data has been downloaded up too, or before this date.'
+                )
 
     def _update_plot(self, data_source: AbstractDataSource, timestamp: pd.Timestamp):
         self.update_plots(data_source, timestamp, plot_type=PlotType.CANDLE_PLOTS)
@@ -479,7 +567,11 @@ class Plot:
         if self._replay:
             asyncio.run(self.replay())
         else:
-            self._chart.show(block=True)
+            # load the chart correctly if in a notebook
+            if self.in_notebook:
+                self._chart.load()
+            else:
+                self._chart.show(block=True)
 
     async def replay(self):
         # if replay is on
